import logging
import os
import pathlib

import toml
import yaml

try:
    import ujson as json
except ImportError:
    import json

try:
    FileNotFoundError
except NameError:
    FileNotFoundError = OSError

log = logging.getLogger("vyper.util")


class ConfigParserError(Exception):
    """Denotes failing to parse configuration file."""

    def __init__(self, message, *args):
        self.message = message
        super(ConfigParserError, self).__init__(message, *args)

    def __str__(self):
        return "While parsing config: {0}".format(self.message)


def abs_pathify(in_path):
    log.info("Trying to resolve absolute path to {0}".format(in_path))

    try:
        return pathlib.Path(in_path).resolve()
    except FileNotFoundError as e:
        log.error('Couldn"t discover absolute path: {0}'.format(e))
        return ""


def exists(path):
    try:
        os.stat(str(path))
        return True
    except FileNotFoundError:
        return False


def unmarshall_config_reader(r, d, config_type):
    config_type = config_type.lower()

    if config_type in ["yaml", "yml"]:
        try:
            f = yaml.safe_load(r)
            try:
                d.update(yaml.safe_load(f))
            except AttributeError:  # to read files
                d.update(f)
        except Exception as e:
            raise ConfigParserError(e)

    elif config_type == "json":
        try:
            f = json.loads(r)
            d.update(f)
        except Exception as e:
            raise ConfigParserError(e)

    elif config_type == "toml":
        try:
            try:
                d.update(toml.loads(r))
            except TypeError:  # to read files
                try:
                    d.update(toml.load(r))
                except TypeError:  # to read streams
                    d.update(r)
        except Exception as e:
            raise ConfigParserError(e)

    return d



import os
import threading
import time

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler


class CustomHandler(FileSystemEventHandler):
    current_event = None

    def process(self, event):
        if event.is_directory is not True:
            self.current_event = event

    def on_modified(self, event):
        self.process(event)

    def on_created(self, event):
        self.process(event)

    def get_event(self):
        current_event = self.current_event
        self.current_event = None
        return current_event


class BaseWatcher(object):
    def __init__(self, config_file, v):
        self.handler = CustomHandler()
        self.config_file = config_file
        self.directory = os.path.dirname(os.path.realpath(config_file))
        self.v = v

    def watch_path(self):
        observer = Observer()
        observer.schedule(self.handler, self.directory)
        observer.start()
        try:
            while True:
                event = self.event
                if event is not None and event.src_path == self.config_file:
                    self.v.read_in_config()
                    if self.v._on_config_change is not None:
                        self.v._on_config_change()
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()

    @property
    def event(self):
        return self.handler.get_event()


class ThreadWatcher(BaseWatcher):
    def __init__(self, config_file, v):
        super(ThreadWatcher, self).__init__(config_file, v)
        self.t = threading.Thread(target=self.watch_path)
        self.t.daemon = True

    def start(self):
        self.t.start()


def get_watcher(config_file, v):
    return ThreadWatcher(config_file, v)



from setuptools import setup

setup(
    name="vyper-config",
    version="1.1.0",
    description="Python configuration with (more) fangs",
    url="http://github.com/alexferl/vyper",
    author="Alexandre Ferland",
    author_email="me@alexferl.com",
    license="MIT",
    packages=["vyper"],
    zip_safe=False,
    install_requires=[
        "distconfig3>=1.0.1",
        "toml>=0.10.0",
        "PyYAML>=5.4.1",
        "watchdog>=0.10.0",
    ],
    setup_requires=["pytest-runner>=6.0.0"],
    tests_require=["pytest>=7.1.0"],
    platforms="any",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
